function [CC,RR,list] = acf(m,varargin)
%
% ACF  Autocovariance and autocorrelation function.
%
% Syntax:
%   [C,R,list] = acf(m,...)
% Arguments:
%   C,R numeric; list cellstr; m model
% Options:
%   'applyto' cellstr|char|Inf (Inf)
%   'filter' char (empty)
%   'nfreq' numeric (256)
%   'order' numeric (0) Maximum order of autocovariance and autocorrelation computed.
%   'select' cellstr|char|Inf (Inf) Select subset of variables.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if nargin > 1 && isnumeric(varargin{1})
  warning('iris:obsolete','This is an obsolete ACF syntax.');
  options.order = varargin{1};
  varargin(1) = [];
end

default = {
  'applyto',Inf,...
  'filter','',...
  'nfreq',256,...
  'order',0,...
  'select',Inf,...
};
options = passopt(default,varargin{:});

if ischar(options.applyto) && ~isempty(options.applyto)
  options.applyto = {options.applyto};
end

if ischar(options.select), options.select = {options.select};
  elseif isempty(options.select), options.select = {''}; end

% function body ---------------------------------------------------------------------------------------------

if isempty(m.solution{1}), error_(12); end

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
CC = nan([ny+nx,ny+nx,options.order+1,nalt]);
if nargout > 1, RR = nan([ny+nx,ny+nx,options.order+1,nalt]); end

% linear filter applied to some of variables
if isinf1(options.applyto)
  applyto = true([1,ny+nx]);
elseif ~isempty(options.applyto)
  applyto = regexprep(options.applyto,'log\((.*?)\)','$1');
  applyto = regexprep(applyto,'\{.*?\}','');
  applyto = findnames(m.name(m.nametype == 1 | m.nametype == 2),options.applyto);
  index = isnan(applyto);
  if any(index), warning_(1,options.applyto(index)); end
  applyto = applyto(~index);
  aux = applyto;
  id = real([m.solutionid{1:2}]);
  applyto = false([1,ny+nx]);
  for i = vech(aux), applyto = applyto | id == i; end
else
  applyto = false([1,ny+nx]); 
end

if ~isempty(options.filter) && any(applyto)
  isfilter = true;
  [frq,filter] = fdfilter(options.nfreq,options.filter);
else
  isfilter = false;
end

% autocovariance function
[flag,index] = isnan(m,'solution');
for ialt = find(~index)
  [T,R,K,Z,H,D,U,Omega] = sspace_(m,ialt);
  if isfilter == true
    % we should multiply by 2*width == 2*pi/nfreq but we skip dividing S by 2*pi in XSF and hence skip multiplying it by 2*pi here
    CC(:,:,:,ialt) = real(xsf(T,R,K,Z,H,D,U,Omega,frq,filter,applyto,options.order))/length(frq);
  else
    CC(:,:,:,ialt) = acovf(T,R,K,Z,H,D,U,Omega,m.eigval(1,:,ialt),options.order);
  end
end

% autocorrelation function
if nargout > 1, RR = cov2corr(CC); end

% list of variables
if nargout > 2 || iscellstr(options.select)
  ny = size(Z,1);
  nx = size(T,1);
  list = ref(printid_(m),1:ny+nx);
end

% select variables
if iscellstr(options.select)
  [CC,index,list,notfound] = select(CC,list,options.select);
  if nargout > 1, RR = RR(index,index,:,:); end
  if ~isempty(notfound), warning_(1,notfound); end
end

end % of primary function -----------------------------------------------------------------------------------