function varargout = get(m,varargin)
%
% GET  Get attributes of model.
%
% Syntax:
%   [value1,value2,...] = get(m,attrib1,attrib2,...)
% Arguments:
%   value depends on attrib; m model; attrib char
% Attributes:
%   'diffuse' returns struct
%   'eigval' returns double
%   'forward' returns double
%   'labels' returns cell
%   'nalt' returns numeric
%   'omega' returns numeric
%   'parameters' returns struct
%   'required' returns cellstr
%   'ssgrowth' returns struct
%   'sslevel' returns struct
%   'sstate' returns struct
%   'stationary' returns struct
%   'std' returns struct
%   'ylist','xlist','elist','plist' returns cellstr
%   'ycomments','xcomments','ecomments', 'pcomments' returns cellstr
%
% The IRIS Toolbox 2007/02/09. Copyright 2007 Jaromir Benes.

if nargin-1 ~= nargout && ~(nargin-1 == 1 && nargout == 0), error('Incorrect number of input or output arguments.'); end
if ~iscellstr(varargin), error('Incorrect type of input argument(s).'); end

% function body ---------------------------------------------------------------------------------------------

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);

varargout = {};
unrecognized = {};

% transform 3D ASSIGN array into 2D cell array to be used in struct
num2cell_ = @(x) num2cell(permute(x,[2,3,1]),2);

for i = 1 : length(varargin)

  x = [];
  attribute = strtrim(varargin{i});

  if length(attribute) > 1 && attribute(1) == '.'
    index = findnames(m.name,attribute(2:end));
    if isnan(index), unrecognized{end+1} = varargin{i};
      else x = m.assign(1,index,:); end

  else

    switch lower(attribute)

    case {'sstate','ss'}
      x = cell2struct(num2cell_(m.assign),vec(m.name),1);
      
    case {'level','sstatelevel','sslevel'}
      x = cell2struct(num2cell_(real(m.assign)),vec(m.name),1);

    case {'growth','sstategrowth','ssgrowth'}
      aux = imag(m.assign);
      % allow for obsolete stationary steady state of log variables
      tmp = m.log(1,:,ones([1,nalt]));
      aux(tmp == true & aux == 0) = 1;
      x = cell2struct(num2cell_(aux),vec(m.name),1);
            
    case {'pars','parameters'}
      index = m.nametype == 4;
      x = cell2struct(num2cell_(m.assign(1,index,:)),vec(m.name(index)),1);

    case {'std'}
      index = strmatch('std_',m.name);
      x = cell2struct(num2cell_(m.assign(1,index,:)),m.name(index),1);
              
    case {'eig','eigval'}
      x = eig(m);

    case {'name','names'}
      x = cell([1,4]);
      for i = 1 : 4, x{i} = m.name(m.nametype == i); end

    case {'yname','ynames','xname','xnames','ename','enames','pname','pnames','elist','xlist','ylist'}
      index = find(attribute(1) == 'yxep');
      x = m.name(m.nametype == index);

    case {'ycomment','ycomments','xcomment','xcomments','ecomment','ecomments','pcomment','pcomments'}
      index = find(attribute(1) == 'yxep');
      x = m.namelabel(m.nametype == index); 
      
    case {'yeqtn','xeqtn','yeqtns','xeqtns','yequations','xequations'}
      index = find(attribute(1) == 'yx');
      x = m.eqtn(m.eqtntype == index);

    case 'log'
      x = struct();
      for i = 1 : length(m.name)
        x.(m.name{i}) = m.log(i);
      end

    case {'eqtn','equation','equations'}
      x = {m.eqtn(m.eqtntype == 1),m.eqtn(m.eqtntype == 2)};

    case {'label','labels'}
      x = {m.eqtnlabel(m.eqtntype == 1),m.eqtnlabel(m.eqtntype == 2)};

    case {'xlabels','ylabels'}
      index = find(attribute(1) == 'yx');
      x = m.eqtnlabel(m.eqtntype == index);
      
    case {'comment','comments'}
      x = cell2struct(m.namelabel,m.name,2);
      
    case {'diffuse','nonstationary','stationary'}
      if isempty(m.solution{1}), error_(12); end
      id = [m.solutionid{:}];
      t0 = imag(id) == 0;
      name = m.name(real(id(t0)));
      [void,index] = isnan(m,'solution');
      flag = nan([sum(t0),nalt]);
      for ialt = find(~index)
        [dy,df,db] = isdiffuse(m.eigval(1,:,ialt),m.solution{4}(:,:,ialt),m.solution{1}(1:nf,:,ialt),m.solution{7}(:,:,ialt));
        d = [dy,df,db,false([1,ne])];
        if strcmp(attribute,'stationary'), flag(:,ialt) = double(~d(t0));
          else flag(:,ialt) = double(d(t0)); end
      end
      x = cell2struct(num2cell(flag,2),vec(name),1);
   
    case {'icond','initcond','required','states','maxlag'}
      if isempty(m.solution{1})
        id = m.systemid{2};
        id(imag(id) >= 0) = [];
        if strcmp(attribute,'maxlag')
          x = min(imag(id));
        else
          id = id + 1i;
          x(1:nalt) = {printid(m.name(real(id)),imag(id)-iff(strcmp(attribute,'states'),0,1))};
          if nalt == 1, x = x{1}; end
        end
      else
        id = m.solutionid{2}(nf+1:end);
        realsmall = getrealsmall();
        if strcmp(attribute,'maxlag')
          x = min(imag(id)) - 1;
        else
          for ialt = 1 : nalt
            index = any(abs(m.solution{1}(:,:,ialt)/m.solution{7}(:,:,ialt)) > realsmall,1);
            x{ialt} = printid(m.name(real(id(index))),imag(id(index))-iff(strcmp(attribute,'states'),0,1));
          end
          if nalt == 1, x = x{1}; end
        end
      end

    case {'forward'}
      if isempty(m.solution{1}), error_(12); end
      nt = sum(m.eqtntype == 2);
      x = size(m.solution{2},2)/nt - 1;
      
    case {'covmat','omega'}
      x = omega_(m);

    case {'stdvec'}
      x = m.assign(1,end-sum(m.nametype == 3)+1:end,:);
         
    case {'sspace'}
      if isempty(m.solution{1}), error_(12); end
      [x{1:8}] = sspace_(m);

    case {'nalt'}
      x = size(m.assign,3);

    case 'outside'
      x = m.outside;

    case 'mapoftunes'
      x = m.mapoftunes;

    case 'precision'
      x = m.precision;

    case 'userdata'
      x = m.userdata;

    otherwise
      index = findnames(m.name,attribute);
      if isnan(index), unrecognized{end+1} = varargin{i};
        else x = m.assign(1,index,:); end

    end

  end

  varargout{end+1} = x;

end

if ~isempty(unrecognized), error('Unrecognised attribute: ''%s''.\n',unrecognized{:}); end

end % of primary function -----------------------------------------------------------------------------------