function [source,outputstd,srse,C] = hardcond(m,source,hard,varargin)
%
% HARDCOND  Impose hard conditions on simulation.
%
% Syntax:
%   [outmean,outstd,srse,C] = hardcond(m,source,range,hard)
% Arguments:
%   outmean struct|cell; outstd struct|cell; srse numeric; C numeric; m model; source cell; range numeric; hard struct
% Options:
%   'output' char ('dbase') Output is returned as database ('dbase') or datapack ('dpack').
%   'include' logical (false) If true initial condition is assumed to be included in source data and returned in output data.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~iscell(source) || ~isstruct(hard) || (nargin > 3 && ~iscellstr(varargin(1:2:end)))
  error('Incorrect type of input argument(s).');
end

default = {...
  'include',false,...
  'output','dbase',...
};
options = passopt(default,varargin{:});

% function body ---------------------------------------------------------------------------------------------

if options.include == true
  for i = 1 : 4
    init{i} = source{i}(:,1,:);
    source{i} = source{i}(:,2:end,:);
  end
end

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
range = source{4};
nper = length(range);
ndata = size(source{2},3);

% number of model/data alternatives
if ndata ~= nalt && nalt ~= 1, error_(44); end % # data and # parameterizations must match
nloop = ndata;

% hard tunes datapack
hard = db2dpack(m,hard,range,[1,2],true,false);
ishard = any(~isnan([hard{1};hard{2}]),1);
k = find(ishard,1,'last');
k0 = size(m.solution{2},2)/ne;
hardindex = {vec(~isnan(hard{1})),vec(~isnan(hard{2}))};
nhard = sum(hardindex{1}) + sum(hardindex{2});

% pre-allocate output datapack
outputstd = {nan([ny,nper,nloop],m.precision),nan([nx,nper,nloop],m.precision),nan([ne,nper,nloop],m.precision),range,meta(m)};
C = {nan([ny,ny,nper,nloop],m.precision),nan([nx,nx,nper,nloop],m.precision)};

% index of available solutions
[flag,nansolution] = isnan(m,'solution');
for iloop = 1 : nloop
  ialt = min([iloop,nalt]);
  if nansolution(ialt) == true, continue, end
  if ialt == iloop
    T = m.solution{1}(:,:,ialt);
    R = m.solution{2}(:,:,ialt);
    K = m.solution{3}(:,:,ialt);
    Z = m.solution{4}(:,:,ialt);
    H = m.solution{5}(:,:,ialt);
    D = m.solution{6}(:,:,ialt);
    U = m.solution{7}(:,:,ialt);
    if k > k0, expand_(); else R = R(:,1:ne*k); end
    stdvec = m.assign(1,end-ne+1:end,ialt);
    stdmat = repmat(stdvec(ones([1,nhard]),:),[1,k]);
    % M{}: impact matrix on [y] and [xu;xp] of expectations
    % N{}: impact matrix on [y] and [xu;xp] of residuals
    % N_: impact matrix on [xu;alpha] of residuals
    [M,N,N_] = impact_();
  end
  isource = {source{1}(:,:,iloop),source{2}(:,:,iloop)};
  discrep = [hard{1}(hardindex{1});hard{2}(hardindex{2})] - [isource{1}(hardindex{1});isource{2}(hardindex{2})];
  MM = [M{1}(hardindex{1},:);M{2}(hardindex{2},:)];
  MM = MM.*stdmat;
  eindex = any(MM,1);
  MM = MM(:,eindex);
  Ee = zeros([1,ne*k]);
  % compute normalised residuals
  Ee(eindex) = transpose(MM)*((MM*transpose(MM))\discrep);
  % square root of the sum of squared normalised residuals (= weighted sum of original residuals)
  srse = sqrt(Ee(eindex)*transpose(Ee(eindex))/sum(eindex));
  Ee = reshape(Ee,[ne,k]);
  tmp = transpose(stdvec(ones([1,k]),:));
  % de-normalise residuals
  Ee(:,1:k) = tmp.*Ee(:,1:k);
  correct = {zeros([ny,nper],m.precision),zeros([nx,nper],m.precision),zeros([ne,nper],m.precision)};
  for t = 1 : nper
    if t > 1, correct{2}(:,t) = T*correct{2}(nf+1:end,t-1); end
    correct{2}(:,t) = correct{2}(:,t) + R(:,1:ne*(k-t+1))*vec(Ee(:,t:k));
    correct{1}(:,t) = Z*correct{2}(nf+1:end,t);
    if t <= k, correct{1}(:,t) = correct{1}(:,t) + H*Ee(:,t); end
  end
  correct{2}(nf+1:end,:) = U*correct{2}(nf+1:end,:);
  correct{3}(:,1:k) = Ee;
  for i = 1 : 3, source{i}(:,:,ialt) = source{i}(:,:,ialt) + correct{i}(:,:); end
  NN = [N{1}(hardindex{1},:);N{2}(hardindex{2},:)];
  NN = NN.*stdmat;
  eindex = vec(any(NN,1));
  NN = NN(:,eindex);
  Ce = eye(ne*k);
  Ce(eindex,eindex) = Ce(eindex,eindex) - transpose(NN)*((NN*transpose(NN))\NN);
  tmp = repmat(stdvec(ones([1,ne*k]),:),[1,k]);
  tmp = tmp.*transpose(tmp);
  Ce = tmp.*Ce;
  estd = sqrt(diag(Ce));
  for t = 1 : k
    C{2}(:,:,t,iloop) = N_((t-1)*nx+(1:nx),1:ne*t)*Ce(1:ne*t,1:ne*t)*transpose(N_((t-1)*nx+(1:nx),1:ne*t));
    if ny > 0
      C{1}(:,:,t,iloop) = N{1}((t-1)*ny+(1:ny),1:ne*t)*Ce(1:ne*t,1:ne*t)*transpose(N{1}((t-1)*ny+(1:ny),1:ne*t));
      outputstd{1}(:,t,iloop) = sqrt(cutsmall(diag(C{1}(:,:,t,iloop))));
    end
    outputstd{3}(:,t,iloop) = estd((t-1)*ne+(1:ne));
  end
  Sigmax = R(:,1:ne)*diag(stdvec.^2)*transpose(R(:,1:ne));
  Sigmay = H*diag(stdvec.^2)*transpose(H);
  for t = k+1 : nper
    if t == 1, C{2}(:,:,t,iloop) = Sigmax;
      else C{2}(:,:,t,iloop) = T*C{2}(nf+1:end,nf+1:end,t-1,iloop)*transpose(T) + Sigmax; end
    if ny > 0, 
      C{1}(:,:,t,iloop) = Z*C{2}(nf+1:end,nf+1:end,t,iloop)*transpose(Z) + Sigmay;
      outputstd{1}(:,t,iloop) = sqrt(cutsmall(diag(C{1}(:,:,t,iloop))));
    end
    outputstd{3}(:,t,iloop) = stdvec;
  end
  for t = 1 : nper
    C{2}(nf+1:end,:,t,iloop) = U*C{2}(nf+1:end,:,t,iloop);
    C{2}(:,nf+1:end,t,iloop) = C{2}(:,nf+1:end,t,iloop)*transpose(U);
    outputstd{2}(:,t,iloop) = sqrt(cutsmall(diag(C{2}(:,:,t,iloop))));
  end
end

if options.include == true
  for i = 1 : 4
    source{i} = [init{i},source{i}];
    if i < 4, outputstd{i} = [zeros([size(outputstd{i},1),1,size(outputstd{i},3)]),outputstd{i}]; end
  end
  outputstd{4} = [init{4},outputstd{4}];
end

% datapack to database conversion if required
if strcmp(options.output,'dbase')
  source = addpardb_(m,dp2dbase(source,'include',options.include));
  outputstd = addpardb_(m,dp2dbase(outputstd,'include',options.include));
end

% end of function body --------------------------------------------------------------------------------------

  function expand_() % nested function ----------------------------------------------------------------------
  R(:,end+(1:ne*(k-k0))) = NaN;
  Xa = m.expand{1}(:,:,ialt);
  Xf = m.expand{2}(:,:,ialt);
  Ru = m.expand{3}(:,:,ialt);
  J = m.expand{4}(:,:,ialt);
  Jk = m.expand{5}(:,:,ialt);
  for i = k0+1 : k
    Ra = -Xa*Jk*Ru; % Jk stores J^(k-1)
    Jk = Jk*J;
    Rf = Xf*Jk*Ru;
    R(1:nf,(i-1)*ne+(1:ne)) = Rf;
    R(nf+1:end,(i-1)*ne+(1:ne)) = Ra;
  end
  end % of nested function ----------------------------------------------------------------------------------

  function [M,N,N_] = impact_() % nested function -----------------------------------------------------------
  % M{}: impact of anticipated residuals on [y] and [xu;xp]
  % N{}: impact of unanticipated residuals on [y] and [xu;xp]
  % N_: impact of unanticipated residuals on [xu;alpha]
  % impact of anticipated residuals
  M{1} = zeros([ny*k,ne*k],m.precision);
  M{2} = zeros([nx*k,ne*k],m.precision);
  % impact of unanticipated residuals
  N{1} = zeros([ny*k,ne*k],m.precision);
  N{2} = zeros([nx*k,ne*k],m.precision);
  M{2}(1:nx,:) = R;
  N{2}(1:nx,1:ne) = R(:,1:ne);
  for t = 2 : k
    nonzero = any(M{2}((t-2)*nx+(1:nx),:),1);
    M{2}((t-1)*nx+(1:nx),nonzero) = T*M{2}((t-2)*nx+(nf+1:nx),nonzero);
    M{2}((t-1)*nx+(1:nx),(t-1)*ne+1:end) = M{2}((t-1)*nx+(1:nx),(t-1)*ne+1:end) + R(:,1:end-(t-1)*ne);
    nonzero = any(N{2}((t-2)*nx+(1:nx),:),1);
    N{2}((t-1)*nx+(1:nx),nonzero) = T*N{2}((t-2)*nx+(nf+1:nx),nonzero);
    N{2}((t-1)*nx+(1:nx),(t-1)*ne+(1:ne)) = N{2}((t-1)*nx+(1:nx),(t-1)*ne+(1:ne)) + R(:,1:ne);
  end
  for t = 1 : k
    nonzero = any(M{2}((t-1)*nx+(nf+1:nx),:),1);
    M{1}((t-1)*ny+(1:ny),nonzero) = Z*M{2}((t-1)*nx+(nf+1:nx),nonzero);
    M{1}((t-1)*ny+(1:ny),(t-1)*ne+(1:ne)) = M{1}((t-1)*ny+(1:ny),(t-1)*ne+(1:ne)) + H;
    M{2}((t-1)*nx+(nf+1:nx),nonzero) = U*M{2}((t-1)*nx+(nf+1:nx),nonzero);
    nonzero = any(N{2}((t-1)*nx+(nf+1:nx),:),1);
    N{1}((t-1)*ny+(1:ny),nonzero) = Z*N{2}((t-1)*nx+(nf+1:nx),nonzero);
    N{1}((t-1)*ny+(1:ny),(t-1)*ne+(1:ne)) = N{1}((t-1)*ny+(1:ny),(t-1)*ne+(1:ne)) + H;
  end
  N_ = N{2};
  for t = 1 : k
    N{2}((t-1)*nx+(nf+1:nx),nonzero) = U*N{2}((t-1)*nx+(nf+1:nx),nonzero);
  end
  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------