function [mloglik,Grad,Hess,A,Pa] = loglikderiv(M,Data,Name,Zd,Evalsstate,varargin)
%
% LOGLIKDERIV  Gradient and Hessian of log-likelihood function.
%
% Syntax:
%   [mloglik,Grad,Hess,A,Pa] = loglikderiv(m,data,name,Zd,evalsstate,...)
% Arguments:
%   p,mloglik,Grad,Hess,A,Pa numeric; m model; data cell; name cellstr; Zd numeric; evalsstate function_handle
% Options:
%   See MODEL/LOGLIK options
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'deterministic',{},...
  'relative',true,...
};
options = passopt(default,varargin{:});

% no deterministic regressors
if nargin < 4, Zd = []; end

% no steady state file
if nargin < 5, Evalsstate = []; end

% function body ---------------------------------------------------------------------------------------------

[ny,nx,nf,nb,ne,np,nalt] = dim_(M);
% multiple parameterizations not allowed
if nalt > 1, error_(47,{'LOGLIKDERIV'}); end
np = length(Name);
[void,nper,ndata] = size(Data{1});
ixobserved = ~isnan(Data{1});

% take current parameterisation
p = M.assign(1,findnames(M.name,Name),:);

iF = zeros(ny);
mloglik = zeros([1,ndata]);
se2 = zeros([1,ndata]);
Grad = zeros([1,np,ndata]);
Hess = zeros([np,np,ndata]);
A = zeros([size(Zd,2),1,ndata]);
Pa = zeros([size(Zd,2)*[1,1],ndata]);
step = max([abs(p);ones([1,np])],[],1)*eps^(1/4);
mm = cell([1,np]);
pm = cell([1,np]);

for idata = 1 : ndata
  disp(idata);
  data_ = transpose(Data{1}(:,:,idata));

  if idata == 1, [mloglik(idata),se2(idata),F,pe,A(:,1,idata),Pa(:,:,idata),setup] = loglik(M,data_,Zd,{},varargin{:});
    else, [mloglik(idata),se2(idata),F,pe,A(:,1,idata),Pa(:,:,idata)] = loglik(M,data_,Zd,setup,varargin{:}); end

  if nargout == 1, continue, end

  F_pe = zeros([ny,1]);
  X = zeros(ny);

  for t = 1 : nper
    index = ixobserved(:,t);
    if idata == 1 || options.relative == true, iF(index,index,t) = inv(F(index,index,t)); end
    F_pe(index,t) = iF(index,index,t)*pe(index,t);
    X(index,index,t) = eye(sum(index)) - F_pe(index,t)*transpose(pe(index,t));
  end

  dF(1:np) = {[]};
  dpe(1:np) = {[]};
  for i = 1 : np    
    if idata == 1
      pm{i} = assign(M,Name{i},p(i)+step(i));
      if ~isempty(Evalsstate)
        aux = Evalsstate(pm{i});
        pm{i} = assign(pm{i},aux);
      end
      pm{i} = solve(pm{i},'persist',true);
      [pmloglik,pse2,pF,ppe,pA,pPa,psetup{i}] = loglik(pm{i},data_,Zd,{},varargin{:});
    else
      [pmloglik,pse2,pF,ppe] = loglik(pm{i},data_,Zd,psetup{i},varargin{:});
    end
    if idata == 1
      mm{i} = assign(M,Name{i},p(i)-step(i));
      if ~isempty(Evalsstate)
        aux = Evalsstate(mm{i});
        mm{i} = assign(mm{i},aux);
      end
      mm{i} = solve(mm{i},'persist',true);
      [mmloglik,mse2,mF,mpe,mA,mPa,msetup{i}] = loglik(mm{i},data_,Zd,{},varargin{:});
    else
      [mmloglik,mse2,mF,mpe] = loglik(mm{i},data_,Zd,msetup{i},varargin{:});
    end
    dF{i} = (pF(:,:,:)-mF(:,:,:))/(2*step(1,i));
    dpe{i} = (ppe(:,:)-mpe(:,:))/(2*step(1,i));
  end

  F_dF(1:np) = {[]};
  F_dpe(1:np) = {[]};
  for i = 1 : np
    for t = 1 : nper
      index = ixobserved(:,t);
      F_dF{i}(index,index,t) = iF(index,index,t)*dF{i}(index,index,t);
      F_dpe{i}(index,t) = iF(index,index,t)*dpe{i}(index,t);
    end
  end

  for t = 1 : nper
    dF_ = [];
    dpe_ = [];
    index = ixobserved(:,t);
    for i = 1 : np
      dF_ = [dF_,vec(dF{i}(index,index,t))];
      dpe_ = [dpe_,dpe{i}(index,t)];
    end
    Grad(1,:,idata) = Grad(1,:,idata) + vech(iF(index,index,t)*transpose(X(index,index,t)))*dF_/2 + transpose(F_pe(index,t))*dpe_; 
  end

  if nargout == 2, continue, end

  for i = 1 : np
    for j = 1 : i
      for t = 1 : nper
        index = ixobserved(:,t);
        % Hess(i,j,idata) = Hess(i,j,idata) + 0.5*trace(F_dF{i}(index,index,t)*F_dF{j}(index,index,t)) + (transpose(dpe{i}(index,t))*F_dpe{j}(index,t));
        % trace A*B = vech(A')*vec(B)
        Xi = transpose(F_dF{i}(index,index,t));
        Xi = transpose(Xi(:));
        Xj = F_dF{j}(index,index,t);
        Xj = Xj(:);
        Hess(i,j,idata) = Hess(i,j,idata) + Xi*Xj/2 + (transpose(dpe{i}(index,t))*F_dpe{j}(index,t));
      end
      if j < i, Hess(j,i,idata) = Hess(i,j,idata); end
    end
  end

end % of for idata = 1 : ndata

end % of primary function -----------------------------------------------------------------------------------