function output = resample(m,source,range,ndraw,varargin)
%
% RESAMPLE  Simulate model drawing from shock distribution.
%
% Syntax:
%   output = resample(m,source,range,ndraw,...)
% Arguments:
%   output cell; m model; source cell; range numeric; ndraw numeric
% Options:
%   'include' logical (false) Include or not initial conditions into the output datapack.
%   'randomise' logical (true) Randomise or not the initial conditions.
%   'tolerance' numeric (1e-5) Tolerance for resampling the initial condition.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {...
  'include',false,...
  'randomize',[],...
  'randomise',true,...
  'tolerance',1e-5,...
};
options = passopt(default,varargin{:});

if ~isempty(options.randomize), options.randomise = options.randomize; end

% function body ---------------------------------------------------------------------------------------------

range = range(1) : range(end);
nper = length(range);
realsmall = getrealsmall();
[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
if nalt > 1, error('Not available for models with more than 1 parameterizations.'); end

[T,R,K,Z,H,D,U,Omega] = sspace_(m);
nunit = sum(abs(abs(m.eigval)-1) <= realsmall);  
nstable = nb - nunit;
Tf = T(1:nf,:);
Ta = T(nf+1:end,:);
Kf = K(1:nf);
Ka = K(nf+1:end);
Rf = R(1:nf,:);
Ra = R(nf+1:end,:);

% stable part of alpha vector
Ta2 = Ta(nunit+1:end,nunit+1:end);
Ra2 = Ra(nunit+1:end,:);
Ea2 = (eye(nstable) - Ta(nunit+1:end,nunit+1:end)) \ Ka(nunit+1:end);
Ca2 = acovf(Ta(nunit+1:end,nunit+1:end),Ra(nunit+1:end,:),[],[],[],[],[],Omega,m.eigval(nunit+1:end),0);

% find sufficient number of iterations to reproduce covariance matrix
if options.randomise == true
  Sigmaa2 = Ra2*Omega*transpose(Ra2);
  Ca2_ = Sigmaa2;
  count = 0;
  while maxabs(Ca2-Ca2_) > options.tolerance
    Ca2_ = Ta2*Ca2_*transpose(Ta2) + Sigmaa2;
    count = count + 1;
  end
end

% draw and scale shocks
stdvec = vec(m.assign(end-sum(m.nametype == 3)+1:end));
stdvec = stdvec(:,ones([1,nper]));
e = randn([ne,nper,ndraw]);
for idraw = 1 : ndraw, e(:,:,idraw) = stdvec.*e(:,:,idraw); end

output = {nan([ny,nper,ndraw]),nan([nx,nper,ndraw]),e,range,meta(m)};
if options.include == true, init = nan([nx,1,ndraw]); end
for idraw = 1 : ndraw
  a20 = Ea2;
  if options.randomise == true, a20 = a20 + initcond_(Ta2,Ra2,Omega,count); end
  a0 = [zeros([nunit,1]);a20];
  output{2}(:,1,idraw) = T*a0 + R*output{3}(:,1,idraw) + K;
  output{1}(:,1,idraw) = Z*output{2}(nf+1:end,1,idraw) + H*output{3}(:,1,idraw) + D;
  for t = 2 : nper
    output{2}(:,t,idraw) = T*output{2}(nf+1:end,t-1,idraw) + R*output{3}(:,t,idraw) + K;
    output{1}(:,t,idraw) = Z*output{2}(nf+1:end,t,idraw) + H*output{3}(:,t,idraw) + D;
  end
  output{2}(nf+1:end,:,idraw) = U*output{2}(nf+1:end,:,idraw);
  if options.include == true, init(nf+1:end,1,idraw) = U*a0; end
end

if options.include == true
  output{1} = [nan([ny,1,ndraw]),output{1}];
  output{2} = [init,output{2}];
  output{3} = [nan([ne,1,ndraw]),output{3}];
  output{4} = [range(1)-1,output{4}];
end

end % of primary function -----------------------------------------------------------------------------------

  function a0 = initcond_(T,R,Omega,n) % subfunction --------------------------------------------------------
  % randomise initial condition by iterating Lyapunov equation
  w = sqrt(diag(Omega));
  ne = size(R,2);
  a0 = R*(w.*randn([ne,1]));
  for i = 1 : n, a0 = T*a0 + R*(w.*randn([ne,1])); end
  end % of subfunction --------------------------------------------------------------------------------------