function [data,varargout] = simulate(m,data,range,varargin)
%
% SIMULATE  Simulate model.
%
% Syntax:
%   output = simulate(m,source,range,...)
% Arguments:
%   output struct|cell; m model; source struct|cell; range numeric
% Options:
%   'anticipate' logical (true)
%   'checkonly' logical (false)
%   'contributions' logical (false)
%   'deviation' logical (false)
%   'include' logical (false)
%   'output' 'auto'|'dbase'|'dpack' ('auto')
%   'ignoreresiduals' logical (false)
%   'plan' plan (empty)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

if ~(isstruct(data) || iscell(data)) || ~isnumeric(range) || ~iscellstr(varargin(1:2:end))
  error('Incorrect type of input argument(s).');
end

default = {...
  'anticipate',true,...
  'checkonly',false,...
  'contributions',false,...
  'deviation',false,...
  'include',false,...
  'output','auto',...
  'ignoreresiduals',false,...
  'plan',[],...
};
options = passopt(default,varargin{:});

% function body ---------------------------------------------------------------------------------------------

% check if solution has been computed
if ~issolved(m), error_(12); end

[ny,nx,nf,nb,ne,np,nalt] = dim_(m);
range = range(1) : range(end);
nper = length(range);
realsmall = getrealsmall();

% create source datapack
if isstruct(data)
  if strcmp(options.output,'auto'), options.output = class(data); end
  xbinit = db2dpack(m,data,range(1)-1,2);
  xbinit = xbinit{2}(nf+1:end,1,:);
  data = db2dpack(m,data,range(1):range(end));
elseif iscell(data)
  if ~rngcmp(data{4},range(1)-1:range(end)), data = dpresize(data,range(1)-1:range(end)); end
  xbinit = data{2}(nf+1:end,1,:);
  for i = 1 : 4, data{i} = data{i}(:,2:end,:); end
  if strcmp(options.output,'auto'), options.output = 'dpack'; end
end

if options.ignoreresiduals == true, data{3}(:) = 0; end

% find endogenised/exogenised data points and create anchors datapack
if isstruct(options.plan)
  anchors = db2dpack(m,options.plan,range,1:3,false);
elseif isplan(options.plan)
  anchors = plan2dpack(options.plan,range);
elseif iscell(options.plan)
  if ~rngcmp(range,options.plan{4}), options.plan = dpresize(options.plan,range); end
  anchors = options.plan;
else
  anchors = {nan([ny,nper]),nan([nx,nper]),nan([ne,nper]),range};
end
for i = 1 : 3, anchors{i} = ~isnan(anchors{i}); end

% no anchors if there is no exogenised or endogenised data point
if ~any(any([anchors{1};anchors{2}])) || ~any(any(anchors{3}))
  anchors{1}(:) = false;
  anchors{2}(:) = false;
  anchors{3}(:) = false;
end

% check for # exogenised points and # endogenised points at each t
if options.anticipate == false
  % fast exogenised simulation if # exogenised points == # endogenised points at each t
  fast = all(sum([anchors{1};anchors{2}],1) == sum(anchors{3},1));
else
  fast = false;
end

% check initial condition for NaN data points
naninit = chkinit_();
if options.checkonly == false && ~isempty(naninit), error_(25,naninit); end

% check source data for NaN exogenised data points
nanexog = chkexog_();
if options.checkonly == false && ~isempty(nanexog), error_(26,nanexog); end

% stop here if check requested only
if options.checkonly == true
  data = isempty(naninit) && isempty(nanexog);
  varargout{1} = naninit;
  varargout{2} = nanexog;
  return
end

% number of loops
ndata = size(data{2},3);
if (ndata == 1 && nalt >= 1) || ndata == nalt, nloop = nalt;
  elseif nalt == 1, nloop = ndata;
  else error_(44); end

% incompatible options for contributions simulation
if options.contributions == true
  if ~isempty(options.plan), error_(51,'CONTRIBUTIONS PLAN'); end
  if options.include == true, error_(51,'CONTRIBUTIONS INCLUDE'); end
  if nalt > 1, error_(47,{'SIMULATE'}); end
end

% position of last residual
lastresid = find(any(any(data{3} ~= 0,3),1),1,'last');
% position of last endogenised residual
lastendog = find(any(anchors{3},1),1,'last');
% position of last exogenised data point
lastexog = max([0,find(any(anchors{1},1),1,'last'),find(any(anchors{2},1),1,'last')]);

% index of available solutions
[flag,nansol] = isnan(m,'solution');

for iloop = 1 : nloop
  if nalt > 1, imodel = iloop;
    else, imodel = 1; end
  if nansol(imodel) == true
    data{1}(:,:,imodel) = NaN;
    data{2}(:,:,imodel) = NaN;
    continue
  end
  if iloop == 1 || ndata > 1
    e = data{3}(:,:,iloop);
    % replace NaN residuals with zeros
    e(isnan(e)) = 0;
    xbinit_ = xbinit(:,1,iloop);
    % set non-required initial conditions to zero
    xbinit_(~vec(m.icondix) & isnan(xbinit_),1) = 0;
  end
  if iloop == 1 || nalt > 1   
    T = m.solution{1}(:,:,iloop);
    R = m.solution{2}(:,:,iloop);
    K = m.solution{3}(:,:,iloop);
    Z = m.solution{4}(:,:,iloop);
    H = m.solution{5}(:,:,iloop);
    D = m.solution{6}(:,:,iloop);
    U = m.solution{7}(:,:,iloop);
    for i = 1 : length(m.expand), expand{i} = m.expand{i}(:,:,iloop); end
    % current expansion t+k0
    k0 = size(R,2)/ne - 1;
    % expansion needed t+k
    if options.anticipate == true, k = max([0,lastresid-1,lastendog-1]);
      else, k = 0; end
    % expand solution to t+k if needed
    if k > k0, expand_(); end
  end
  if lastexog > 0
    if fast == true
      % fast exogenised simulation
      [MY,MX] = fastimpact_();
      [y,x,e] = fastsimulate_(U\xbinit_,e);
      place = iloop;
    else
      % exogenised simulation other than fast
      % plain simulation first
      [y,x] = simulate_(U\xbinit_,e,lastexog,options.deviation);
      % compute impact matrices and endogenised residuals
      [MY,MX] = impact_();
      adde = exogenise_();
      index = find(adde ~= 0);
      e(index) = e(index) + adde(index); 
      [y,x] = simulate_(U\xbinit_,e,nper,options.deviation);
      place = iloop;

    end
  else
    if options.contributions == false
      % plain simulation
      [y,x] = simulate_(U\xbinit_,e,nper,options.deviation);
      place = iloop;
    else
      % compute contributions of residuals and init.cond.+deterministic
      [y0,x0] = simulate_(U\xbinit_,e,nper,options.deviation);
      e0 = e;
      e = zeros([size(e0),ne+1]);
      for i = 1 : ne
        e(i,:,i) = e0(i,:);
        [y(:,:,i),x(:,:,i)] = simulate_(zeros(size(xbinit_)),e(:,:,i),nper,true);
      end
      [y(:,:,end+1),x(:,:,end+1)] = simulate_(U\xbinit_,e(:,:,end),nper,options.deviation);
      place = 1 : ne+1;
    end
  end
  % save final results
  data{1}(:,:,place) = y;
  data{2}(:,:,place) = x;
  data{3}(:,:,place) = e;
end

if options.include == true
  xbinit(~m.icondix,1,:) = NaN;
  data{1} = [nan([ny,1,nloop],m.precision),data{1}];
  if size(xbinit,3) == 1, xbinit = xbinit(:,:,ones([1,nloop])); end
  data{2} = [[nan([nf,1,nloop],m.precision);xbinit],data{2}];
  data{3} = [nan([ne,1,nloop],m.precision),data{3}];
  data{4} = [range(1)-1,range];
end

if strcmp(options.output,'dbase')
  data = addpardb_(m,dp2dbase(data,'include',options.include));
elseif strcmp(options.output,'struct')
  data = addpardb_(m,dp2dbase(data,'include',options.include,'merge',struct()));
end

% end of function body --------------------------------------------------------------------------------------

  function naninit = chkinit_() % nested function -----------------------------------------------------------
  % check for NaN initial condition
  naninit = {};
  index = any(isnan(xbinit(m.icondix,1,:)),3);
  if any(index)
    id = m.solutionid{2}(nf+1:end);
    id = id(m.icondix);
    naninit = printid(m.name(real(id(index))),imag(id(index))-1);
  end
  end % of nested function ----------------------------------------------------------------------------------

  function nanexog = chkexog_() % nested function -----------------------------------------------------------
  % check for NaN exogenised data points
  nanexog = {};
  index = any([anchors{1};anchors{2}] & [any(isnan(data{1}),3);any(isnan(data{2}),3)],2);
  if any(index)
    id = [m.solutionid{1:2}];
    nanexog = unique(m.name(real(id(index))));
  end
  end % of nested function ----------------------------------------------------------------------------------

  function expand_() % nested function ----------------------------------------------------------------------
  R(:,end+(1:ne*(k-k0))) = NaN;
  Xa = expand{1};
  Xf = expand{2};
  Ru = expand{3};
  J = expand{4};
  Jk = expand{5};
  for i = k0+1 : k
    Ra = -Xa*Jk*Ru; % Jk stores J^(k-1)
    Jk = Jk*J;
    Rf = Xf*Jk*Ru;
    R(1:nf,i*ne+(1:ne)) = Rf;
    R(nf+1:end,i*ne+(1:ne)) = Ra;
  end
  end % of nested function ----------------------------------------------------------------------------------

  function [MY,MX] = impact_() % nested function ------------------------------------------------------------
  MY = zeros([ny*lastexog,ne*lastendog],m.precision);
  MX = zeros([nx*lastexog,ne*lastendog],m.precision);
  if options.anticipate == true
    RR = R(:,1:ne*lastendog);
    MX(1:nx,:) = RR;
  else
    RR = R(:,1:ne);
    MX(1:nx,1:ne) = RR;
  end
  for t = 2 : lastexog
    nonzero = find(any(MX((t-2)*nx+(1:nx),:),1));
    MX((t-1)*nx+(1:nx),nonzero) = T*MX((t-2)*nx+(nf+1:nx),nonzero);
    if options.anticipate == true
      RR = RR(:,1:end-ne);
      MX((t-1)*nx+(1:nx),(t-1)*ne+1:end) = MX((t-1)*nx+(1:nx),(t-1)*ne+1:end) + RR;
    elseif t <= lastendog
      MX((t-1)*nx+(1:nx),(t-1)*ne+(1:ne)) = MX((t-1)*nx+(1:nx),(t-1)*ne+(1:ne)) + RR;
    end
  end    
  for t = 1 : lastexog
    MY((t-1)*ny+(1:ny),:) = Z*MX((t-1)*nx+(nf+1:nx),:);
    MY((t-1)*ny+(1:ny),(t-1)*ne+(1:ne)) = MY((t-1)*ny+(1:ny),(t-1)*ne+(1:ne)) + H;
    nonzero = find(any(MX((t-1)*nx+(nf+1:nx),:),1));
    MX((t-1)*nx+(nf+1:nx),nonzero) = U*MX((t-1)*nx+(nf+1:nx),nonzero);
  end
  end % of nested function ----------------------------------------------------------------------------------

  function [MY,MX] = fastimpact_() % nested function --------------------------------------------------------
  MX = R(:,1:ne);
  MY = Z*MX(nf+1:end,:) + H;
  MX(nf+1:end,:) = U*MX(nf+1:end,:);
  end % of nested function ----------------------------------------------------------------------------------

  function [y,x] = simulate_(init,e,nt,deviation) % nested function -----------------------------------------
  y = nan([ny,nt],m.precision);
  x = nan([nx,nt],m.precision);
  lastresid_ = find(any(e ~= 0,1),1,'last');
  if options.anticipate == true, Rindex = any(abs(R(:,1:ne*lastresid_)) > 0,1);
    else, Rindex = any(abs(R(:,1:ne)) > 0,1); end
  for t = 1 : nt
    x(:,t) = T*init;
    if deviation == false, x(:,t) = x(:,t) + K; end
    if lastresid_ > 0
      if options.anticipate == true
        aux = vec(e(:,t:t+lastresid_-1));
        Eindex = vech(aux ~= 0);
        x(:,t) = x(:,t) + R(:,Rindex & Eindex)*aux(Rindex & Eindex);
      else
        Eindex = vech(e(:,t) ~= 0);
        x(:,t) = x(:,t) + R(:,Rindex & Eindex)*e(Rindex & Eindex,t);
      end
    end
    lastresid_ = lastresid_ - 1;
    if options.anticipate == true, Rindex = Rindex(1,1:end-ne); end
    init = x(nf+1:end,t);
  end
  if ny > 0
    % measurement variables
    Hindex = any(abs(R(:,1:ne)) > 0,1);
    y = Z*x(nf+1:end,:) + H(:,Hindex)*e(Hindex,1:nt);
    if deviation == false, y = y + D(:,ones([1,nt])); end
  end
  % transformation of alpha into xb
  x(nf+1:end,:) = U*x(nf+1:end,:);
  end % of nested function ----------------------------------------------------------------------------------

  function adde = exogenise_() % nested function ------------------------------------------------------------
  yindex = find(vec(anchors{1}(:,1:lastexog)));
  xindex = find(vec(anchors{2}(:,1:lastexog)));
  eindex = find(vec(anchors{3}(:,1:lastendog)));
  ydiscrep = [];
  xdiscrep = [];
  for t = 1 : lastexog
    aux = anchors{1}(:,t);
    if any(aux), ydiscrep = [ydiscrep;data{1}(aux,t,iloop)-y(aux,t)]; end
    aux = anchors{2}(:,t);
    if any(aux), xdiscrep = [xdiscrep;data{2}(aux,t,iloop)-x(aux,t)]; end
  end
  W = [MY(yindex,eindex);MX(xindex,eindex)];
  adde = zeros([ne,lastendog],m.precision);
  adde(eindex) = W\[ydiscrep;xdiscrep];
  end % of nested function ----------------------------------------------------------------------------------

  function [y,x,e] = fastsimulate_(init,e) % nested function ------------------------------------------------
  y = nan([ny,nper],m.precision);
  x = nan([nx,nper],m.precision);
  lastresid_ = find(any(e ~= 0,1),1,'last');
  Rindex = any(abs(R(:,1:ne)) > 0,1);
  if ny > 0, Hindex = any(abs(R(:,1:ne)) > 0,1); end
  for t = 1 : nper
    x(:,t) = T*init;
    if options.deviation == false, x(:,t) = x(:,t) + K; end
    if lastresid_ > 0, x(:,t) = x(:,t) + R(:,Rindex)*e(Rindex,t); end
    lastresid_ = lastresid_ - 1;
    init = x(nf+1:end,t);
    if ny > 0
      % measurement variables
      y(:,t) = Z*init + H(:,Hindex)*e(Hindex,t);
      if options.deviation == false, y(:,t) = y(:,t) + D; end
    end
    % transformation of alpha into xb
    x(nf+1:end,t) = U*init;
    % fast tunes
    if any(anchors{3}(:,t))
      discrep = [...
        data{1}(anchors{1}(:,t),t,iloop)-y(anchors{1}(:,t),t);...
        data{2}(anchors{2}(:,t),t,iloop)-x(anchors{2}(:,t),t);...
      ];
      adde = [MY(anchors{1}(:,t),anchors{3}(:,t));MX(anchors{2}(:,t),anchors{3}(:,t))]\discrep;
      y(:,t) = y(:,t) + MY(:,anchors{3}(:,t))*adde;
      x(:,t) = x(:,t) + MX(:,anchors{3}(:,t))*adde;
      e(anchors{3}(:,t),t) = e(anchors{3}(:,t),t) + adde;
      init = U\x(nf+1:end,t);
    end
  end
  end % of nested function ----------------------------------------------------------------------------------

end % of primary function -----------------------------------------------------------------------------------