function [m,flag] = sstate(m,varargin)
%
% SSTATE  Compute steady state (and reduced-form solution with linear models).
%
% Syntax:
%   [m,flag] = sstate(m,...)
% Arguments:
%   m: model; flag: logical
% Options:
%   'algorithm': @lsqnonlin|@fsolve (@lsqnonlin) Algorithm to numerically solve for the steady state (non-linear models only).
%   'fix': cellstr (empty) Fix user-supplied steady-state values (non-linear models only).
%   'optimset': struct (see below) Optimization Toolbox options (non-linear models only).
%   'persist': logical (false) Preserve model equations in memory until next FALSE (linear models only).
%   'randomise': numeric ([1,1]) Randomise initial values (non-linear models only).
%
% Optimization Toolbox options with IRIS specific default values:
%    'display': 'final'|'iter'|'notify'|'off' ('iter') Level of display.
%    'largescale': 'on'|'off' ('off') Use large-scale algorithm if possible.
%    'levenbergmarquardt': 'on'|'off' ('off') Chooses Levenberg-Marquardt over Gauss-Newton algorithm.
%    'tolx': numeric (1e-9) Termination tolerance on X.
%    'tolfun': numeric (1e-9) Termination tolerance on the function value.
% See Optimization Toolbox help for default settings of other options.
%
% The IRIS Toolbox 2007/03/09. Copyright 2007 Jaromir Benes.

aux = optimset('display','iter','largescale','off','tolx',1e-12,'tolfun',1e-12,'levenbergmarquardt','off');

default = {
  'algorithm',@lsqnonlin,...
  'display',[],...
  'fix',{},...
  'maxiter',[],...
  'maxfunevals',[],...
  'optimset',struct,...
  'persist',false,...
  'randomise',[1,1],...
};
options = passopt(default,varargin{1:end});

options.optimset = optimset(aux,options.optimset);

if ~isempty(options.display)
  if ischar(options.display)
    options.optimset = optimset(options.optimset,'display',options.display);
  elseif islogical(options.display)
    options.optimset = optimset(options.optimset,'display',iff(options.display,'iter','off'));
  end
end

if ~isempty(options.maxiter)
  options.optimset = optimset(options.optimset,'maxiter',options.maxiter);
end

if ~isempty(options.maxfunevals)
  options.optimset = optimset(options.optimset,'maxfunevals',options.maxfunevals);
end

if ischar(options.fix), options.fix = {options.fix}; end

% function body ---------------------------------------------------------------------------------------------

if m.linear == false
  [flag,list] = isnan(m,'parameters');
  if flag == true, warning_(2,list); end
else
  m = solve(m,'persist',options.persist);
end
[m,flag] = sstate_(m,options);      

end % of primary function -----------------------------------------------------------------------------------