function [w,dpack,restrict,count] = rvar(varargin)
%
% RVAR  Estimate reduced-form VAR model.
%
% Syntax:
%   [w,dpack,constr] = rvar(y,...)
%   [w,dpack,constr] = rvar(x,range,...)
%   [w,dpack,constr] = rvar(d,list,range,...)
% Arguments
%   w rvar; dpack cell; constr cell; y numeric|cell; x tseries; range numeric; list cellstr|char; d struct
% Options:
%   'order' numeric (1) Order of VAR
%   'polyorder' numeric|Inf (Inf) Order of polynomial
%   'cointeg' numeric (empty) Matrix of conintegrating vectors
%   'constraint' char|cell (empty) Linear constraints on parameters
%   'comment' ? (empty) Comments
%   'constant' logical (true) If false constant is dropped from regression
%   'maxiter' numeric (10) Maximum number of iteration for FGLS
%   'mean' numeric (empty) Constraint on process mean
%   'tolerance' numeric (1e-5) Tolerance criterion for FGLS
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

[data,options] = deal(cell([1,0]));
if nargin == 0
  data = [];
  options = {};
elseif nargin == 1 && isstruct(varargin{1})
  w = class(empty_(),'rvar'); 
  for f = vech(fieldnames(w))
    try, w.(f{1}) = varargin{1}.(f{1}); catch, end
  end
  return
elseif isnumeric(varargin{1})
  data = varargin(1);
  options = varargin(2:end);
elseif istseries(varargin{1})
  data = varargin(1:2);
  options = varargin(3:end);
elseif isstruct(varargin{1})
  data = varargin(1:3);
  options = varargin(4:end);
elseif iscell(varargin{1})
  data = varargin(1);
  options = varargin(2:end);
else
  error('Incorrect type of input argument(s).');
end

default = {
  'order',1,...
  'polyorder',Inf,...
  'cointeg',[],...
  'constraint','',...
  'constraints','',...
  'comment','',...
  'const',[],...
  'constant',true,...
  'maxiter',10,...
  'mean',[],...
  'tolerance',1e-5,...
};
options = passopt(default,options{:});

% adjust range for tseries or dbase input
% range is explained data points
% data to be retrieved need to include pre-sample initial condition
if any(length(data) == [2,3]) && ~isinf1(data{end})
  data{end} = data{end}(1)-options.order:data{end}(end);
end

% get data from specified range
if ~isempty(data), [y,range] = getdata(data{:}); end

% duplicate options
if ~isempty(options.constraints), options.constraint = options.constraints; end
if ~isempty(options.const), options.constant = options.const; end
options.mean = vec(options.mean);

% function body ---------------------------------------------------------------------------------------------

w = class(empty_(),'rvar');
restrict(1:2) = {[]};
dpack(1:4) = {[]};

if nargin > 0 
  [ny,void,nalt] = size(y);
  dpack{1} = y;
  dpack{2} = zeros([0,length(range),nalt]);
  dpack{3} = nan(size(y));
  dpack{4} = range;
  [sample,flag] = getsample(y);
  y = y(:,sample,:);
  if flag == false, error_(9); end % within-sample NaNs
  if any(~sample), warning_(4); end % effective estimation sample shorter than user-specified
  if sum(sample) < ny || sum(sample)-options.order <= 0, error_(10); end % insufficient number of observations
  nper = sum(sample) - options.order;
  if ~isempty(options.mean)
    ymean = options.mean(:,ones([1,size(y,2)]));
    for ialt = 1 : nalt, y(:,:,ialt) = y(:,:,ialt) - ymean; end
    options.constant = false;
  end
  % estimate RVAR
  [w.A,w.K,w.Omega,w.nhyper,w.resid,restrict{1:2},count] =...
    glsq_(w,y,options.constant,options.order,options.polyorder,options.constraint,options.cointeg,options.tolerance,options.maxiter);
  dpack{3}(:,sample,:) = [nan([ny,options.order,nalt]),w.resid];
  sample = all(all(~isnan(dpack{3}),3),1);
  w.sample = dpack{4}(sample);
  nper = length(w.sample);
  % back out constant given mean restriction
  for ialt = 1 : nalt
    if ~isempty(options.mean)
      w.K(:,1,ialt) = sum(var2poly(w.A(:,:,ialt)),3)*options.mean;
    end
    % information criteria
    w.aic(ialt) = log(det(w.Omega(:,:,ialt))) + 2/nper * w.nhyper;
    w.sbc(ialt) = log(det(w.Omega(:,:,ialt))) + log(nper)/nper * w.nhyper;
  end
  w.comment = options.comment;
  % compute quasi-triangular form and eigenvalues at the time when state space or eigenvalues are requested
end

  function w = empty_() % nested function -------------------------------------------------------------------
  w.IRIS_VAR = true;
  w.A = [];
  w.B = [];
  w.K = [];
  w.Omega = [];
  w.std = NaN;
  w.TZ = {};
  w.resid = []; % for Portmonteau test
  w.sample = [];
  w.aic = NaN;
  w.sbc = NaN;
  w.nhyper = NaN;
  w.eigval = [];
  w.comment = '';
  end % of nested function ----------------------------------------------------------------------------------
  
end