function output = simulate(w,source,range,varargin)
%
% SIMULATE  Simulate VAR model.
%
% Syntax:
%   output = simulate(w,source,range,...)
% Arguments:
%   output cell; w rvar|svar; source cell; range numeric; B numeric
% Options:
%  'endogenise' numeric (empty)
%  'exogenise' numeric (empty)
%  'deviation' logical (false)
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

[ny,p,nalt] = dim(w);

default = {
  'anchor',[],... % obsolete option
  'exogenise',[],...
  'deviation',false,...
  'disclose',{},... % obsolete option
  'endogenise',[]...
  'free',{},... % obsolete option
};
options = passopt(default,varargin{1:end});

if ~isempty(options.disclose) && isempty(options.endogenise)
  warning('DISCLOSE is an obsolete option. Use ENDOGENISE instead.');
  options.endogenise = options.disclose;
elseif ~isempty(options.free) && isempty(options.endogenise)
  warning('FREE is an obsolete option. Use ENDOGENISE instead.');
  options.endogenise = options.free;
end
if ~isempty(options.anchor) && isempty(options.exogenise)
  warning('ANCHOR is an obsolete option. Use EXOGENISE instead.');
  options.exogenise = options.anchor;
end

% function body ---------------------------------------------------------------------------------------------

realsmall = getrealsmall();
ndata = size(source{1},3);
nloop = max([nalt,ndata]);
d = nloop - ndata;
if d > 0
  source{1} = source{1}(:,:,ndata*ones([1,d]));
  source{3} = source{3}(:,:,ndata*ones([1,d]));
end

nper = length(source{4});
output = source;
if isempty(range)
  return
elseif isinf1(range)
  sample = getsample(source{1});
  start = find(sample,1) + p;
  finish = nper;
else
  start = round(range(1) - source{4}(1)) + 1;
  finish = round(range(end) - source{4}(1)) + 1;
end

% extend output datapack up to end of simulation horizon
output{1}(:,end+1:finish,:) = NaN;
output{2}(:,end+1:finish,:) = 0;
output{3}(:,end+1:finish,:) = 0;
output{4} = [output{4},output{4}(end)+1:range(end)];
% extend plan up to end of simulation horizon
if ~isempty(options.exogenise)
  options.exogenise(:,end+1:finish) = 0;
  options.exogenise = options.exogenise ~= 0 & ~isnan(options.exogenise);
end
if ~isempty(options.endogenise)
  options.endogenise(:,end+1:finish) = 0;
  options.endogenise = options.endogenise ~= 0 & ~isnan(options.endogenise);
end

% initial condition not available
if start <= p, output{1}(:,:,:) = NaN; return
  elseif start > nper+1, return, end

for iloop = 1 : nloop
  if iloop <= nalt
    A = w.A(:,:,iloop); 
    index = any(abs(A) > realsmall,1);
    if isempty(w.B), b = eye(ny);
      else, b = w.B(:,:,iloop); end
    K = w.K(:,:,iloop);
    end
  for t = start : finish
    y = vec(output{1}(:,t-1:-1:t-p,iloop));
    if ~isempty(options.exogenise) && ~isempty(options.endogenise) && any(options.exogenise(:,t)) && any(options.endogenise(:,t))
      % exogenise/endogenise time t
      x = A(:,index)*y(index); + b*output{3}(:,t,iloop);
      if options.deviation == false, x = x + K; end
      addresid = b(options.exogenise(:,t),options.endogenise(:,t)) \ (output{1}(options.exogenise(:,t),t,iloop)-x(options.exogenise(:,t)));
      output{3}(options.endogenise(:,t),t,iloop) = output{3}(options.endogenise(:,t),t,iloop) + addresid;
    end
    % simulate time t
    output{1}(:,t,iloop) = A(:,index)*y(index) + b*output{3}(:,t,iloop);
    if options.deviation == false, output{1}(:,t,iloop) = output{1}(:,t,iloop) + K; end
  end
end

end % of primary function -----------------------------------------------------------------------------------