function [w,dpack,B] = svar(w,dpack,varargin)
%
% SVAR  Identify structural VAR from reduced-form VAR.
%
% Syntax:
%   [w,data,B] = svar(w,dpack,...)
% Arguments: 
%   w var; data cell; B numeric
% Options:
%   'method' char ('chol') Type of the structural identifying scheme.
%   'std' numeric (1) Scale B matrix to get a particular std.error for all shocks.
%   'ordering' numeric (empty) Ordering of variables in triangular identifying schemes.
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

default = {
  'method','chol',...
  'std',1,...
  'ordering',[],...
};
options = passopt(default,varargin{1:end});

% function body ---------------------------------------------------------------------------------------------

[ny,p,nalt] = dim(w);
A = w.A;
Omega = w.Omega;
if ~isempty(options.ordering)
  options.ordering = vech(options.ordering);
  if ~( length(options.ordering) == ny && length(intersect(1:ny,options.ordering)) == ny )
    error_(14);
  end
  A = A(options.ordering,options.ordering,:);
  Omega = Omega(options.ordering,options.ordering,:);
end
B = nan([ny,ny,nalt]);
if isnumeric(options.method)
  B = options.method;
else
  switch lower(options.method)
  case 'chol'
    for ialt = 1 : nalt, B(:,:,ialt) = transpose(chol(Omega(:,:,ialt))); end
  otherwise
    error(['Unrecognized method: ''',options.method,'''.']);
  end
end
if ~isempty(options.ordering)
  [aux,backorder] = sort(options.ordering);
  B = B(backorder,:,:);
end
if length(options.std) > 1, w.std = options.std;
  else, w.std = options.std*ones([1,nalt]); end
for ialt = 1 : nalt
  if w.std(ialt) ~= 1, B(:,:,ialt) = B(:,:,ialt)/w.std(ialt); end
end
if nargin > 1 && ~isempty(dpack)
  ndata = size(dpack{1},3);
  for ialt = 1 : ndata, dpack{3}(:,:,ialt) = B(:,:,ialt)\dpack{3}(:,:,ialt); end
end
w.B = B;

end % of primary function -----------------------------------------------------------------------------------