function d = dbload(fname,varargin)
%
% DBLOAD  Load database from CSV file.
%
% Syntax:
%   d = dbload(fname,...)
% Arguments:
%   d struct; fname char
% Options:
%   'case' 'lower'|'upper'|empty (empty)
%   'freq' 0|1|2|4|12|NaN (NaN)
%   'nan' cellstr|char ({'na','#n/a'})
%
% The IRIS Toolbox 2007/02/03. Copyright 2007 Jaromir Benes.

config = load('irisconfig.mat');
default = {...
  'case','',...
  'changecase','',...
  'freq',NaN,...
  'freqletters',config.freqletters,...
  'nan',{'na','#n/a'},...
};
options = passopt(default,varargin{1:end});

if ~isempty(options.changecase) && isempty(options.case), options.case = options.changecase; end
if ~isa(options.freqletters,'char') || length(options.freqletters) < 3, options.freqletters = config.freqletters; end
if ischar(options.nan), options.nan = {options.nan}; end
options.nan = lower(options.nan);

% function body ---------------------------------------------------------------------------------------------

d = struct();

% read file
text = file2char(fname);
text = strrep(text,char(13),'');

% read headings
name = {};
class = {};
comment = {};
isdate = false;
eol = (text == char(10));
while ~isempty(text) && isdate == false
  pos = find(eol,1);
  if isempty(pos), line = text;
    else line = text(1:pos-1); end
  tokens = regexp(line,'([^",]*),|([^",]*)$|"(.*?)",|"(.*?)"$','tokens');
  tokens = [tokens{:}];
  if isempty(tokens) | all(cellfun(@isempty,tokens)), continue, end
  ident = lower(strtrim(tokens{1}));
  if ~isempty(ident) && ident(1) == '%',
    elseif ~isempty(strfind(ident,'class')), class = tokens(2:end);
    elseif ~isempty(strfind(ident,'comment')), comment = tokens(2:end);
    elseif isempty(ident), name = tokens(2:end);
    elseif ~isempty(ident), isdate = true; end
  if isdate == false
    if ~isempty(pos), text = text(pos+1:end); eol = eol(pos+1:end);
      else text = ''; eol = []; end
  end
end

if length(class) < length(name), comment(length(class)+1:length(name)) = {''}; end
if length(comment) < length(name), comment(length(comment)+1:length(name)) = {''}; end

if ~isempty(text)
  text = lower(text);
  if text(end) ~= char(10), text(end+1) = char(10); end
  if isnan(options.freq), freq = datfreq(str2dat(ident));
    else freq = options.freq; end
  if ~any(freq == [1,2,4,12,0]), error('Unable to determine frequency of dates.',ident); end
  if freq > 0
    letter = lower(options.freqletters(freq == [1,2,4,12]));
    text = strrep(text,[letter,','],',1');
    text = strrep(text,[letter,char(10)],',1');
    text = strrep(text,letter,',');
    text = strrep(text,letter,',');
    text = strrep(text,'?',',');
  end
  for i = 1 : length(options.nan)
    text = regexprep(text,['\<',options.nan{i},'\>'],'nan');
  end
  data = dlmread_(text,',');
  sample = find(all(~isnan(data(:,1:2)),2));
  dates = datcode(data(sample,1),data(sample,2),freq);
  data = data(:,3:end);
else
  dates = [];
  data = [];
end

switch lower(options.case)
case 'lower', name = lower(name);
case 'upper', name = upper(name);
end

nsample = length(sample);
while ~isempty(name)
  if isempty(name{1})
    shift = 1;
  else
    if ~isempty(class), class_ = class{1};
      else class_ = 'tseries'; end
    dim = regexp(class_,'\[\d*\]','match');
    class_ = lower(regexp(class_,'^\w*','match','once'));
    if ~isempty(dim), dim = str2num(['[',sprintf('%s,',dim{:}),']']);
      else, dim = []; end
    if isempty(dim) || ~isnumeric(dim) || any(isnan(dim)), dim = 1; end
    if strcmp(class_,'tseries')
      shift = prod(dim);
      if ~isempty(data)
         data_ = reshape(data(sample,1:shift),[nsample,dim]);
         d.(name{1}) = tseries(dates,data_,comment(1:shift));
       else
         d.(name{1}) = tseries([],zeros([0,dim]));
       end
    else
      shift = 1;
      try
        x = reshape(data(1:prod(dim),1),dim);
        x = eval(sprintf('%s(x)',class_));
        d.(name{1}) = x;
      catch
      end
    end
  end
  data = data(:,shift+1:end);
  name = name(shift+1:end);
  comment = comment(shift+1:end);
  class = class(shift+1:end);
end

end % of primary function -----------------------------------------------------------------------------------